#include "tic_tac_toe.h"
#include "SDL_events.h"
#include "rectangle.h"

#include <SDL.h>
#include <iostream>

TicTacToe::TicTacToe(int width, int height, std::string title) {
    auto throw_error = [](std::string which) {
        throw std::runtime_error(which + " " + SDL_GetError());
    };

    // Alle Komponenten von SDL initialisieren
    // siehe https://wiki.libsdl.org/SDL_Init
    if (SDL_Init(SDL_INIT_EVERYTHING) < 0) {
        throw_error("SDL_Init fehlgeschlagen");
    }

    // Beim beenden SDL_Quit aufrufen (Resourcen freigeben)
    atexit(SDL_Quit);

    // Fenster erstellen, siehe https://wiki.libsdl.org/SDL_CreateWindow
    window.reset(SDL_CreateWindow(
        title.c_str(),
        SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
        width, height,
        SDL_WINDOW_RESIZABLE));

    if (!window) {
        throw_error("SDL_CreateWindow fehlgeschlagen");
    }

    // Renderer erstellen, siehe https://wiki.libsdl.org/SDL_CreateRenderer
    renderer.reset(SDL_CreateRenderer(window.get(), -1, 0));

    if (!renderer) {
        throw_error("SDL_CreateRenderer fehlgeschlagen");
    }

    // Masse speichern
    this->width = width;
    this->height = height;

    // Automatisch skalieren wenn sich Fenstergroesse aendert
	SDL_RenderSetLogicalSize(renderer.get(), width, height);

    // Fenster schwarz ausmalen
	SDL_RenderClear(renderer.get());

    // Aenderung sichtbar machen
	SDL_RenderPresent(renderer.get());

    // Eigener Initialisierungscode ab hier
    circle.reset(new Image(renderer.get(), "circle.bmp"));

    int xoff = 128;
    lines.push_back(Rectangle(Point(xoff + 256, 0), 20, height));
    lines.push_back(Rectangle(Point(xoff + 512, 0), 20, height));
    lines.push_back(Rectangle(Point(xoff + 0, 256), 256 * 3, 20));
    lines.push_back(Rectangle(Point(xoff + 0, 512), 256 * 3, 20));
}

bool TicTacToe::processEvents() {
    // Nutzereingaben verarbeiten
    SDL_Event evt;

    // Events einlesen
    // siehe https://wiki.libsdl.org/SDL_Event
    while (SDL_PollEvent(&evt)) {
        // Jeweils ein Event verarbeiten
        if (evt.type == SDL_QUIT) {
            // Benutzer hat Programmende angefordert
            return false;
        } else if (evt.type == SDL_MOUSEBUTTONDOWN) {
            if (evt.button.button == SDL_BUTTON_LEFT) {
                circle->position.moveTo(
                    evt.button.x - circle->getWidth() / 2,
                    evt.button.y - circle->getHeight() / 2);
            }
        }
    }

    return true;
}

void TicTacToe::update() {
    //Point wsize = getWindowSize();
}

void TicTacToe::render() {
    SDL_SetRenderDrawColor(renderer.get(), 0xFF, 0xFF, 0xFF, 0xFF);
    SDL_RenderClear(renderer.get());

    circle->draw(renderer.get());

    for (auto& line: lines) {
        line.draw(renderer.get());
    }

    SDL_RenderPresent(renderer.get());
}

Point TicTacToe::getWindowSize() {
    return Point(width, height);
}

